@@ -33,7 +33,13 @@ from airflow.exceptions import (
     TaskDeferred,
 )
 from airflow.models import DagBag, DagRun, TaskInstance
-from airflow.models.baseoperator import BaseOperator
+
+from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS
+
+if AIRFLOW_V_3_0_PLUS:
+    from airflow.sdk import BaseOperator
+else:
+    from airflow.models.baseoperator import BaseOperator  # type: ignore[no-redef]
 from airflow.models.dag import DAG
 from airflow.models.serialized_dag import SerializedDagModel
 from airflow.models.xcom_arg import XComArg
